"""Add Motion Regressors.

This script defines a function that takes a design matrix .sdm file as well as
a motion regressors sdm file and creates a new design matrix .sdm file with a
custom suffix (default='_WithMotionRegressors') which combines the original
design matrix with the motion regressors.
Optionally, the motion regressors can be z-transformed.

Example (will create a new file "Design_WithMotionRegressors.sdm"):

    from add_motion_regressors import add_motion_regressors

    add_motion_regressors("Design.sdm", "Run1_3DMC.sdm", ztransform=True)

"""

__author__ = "Florian Krause"
__version__ = "0.1.0"
__date__ = "2017-05-16"


import os

import numpy as np
from scipy.stats.mstats import zscore


def add_motion_regressors(sdm, motion, ztransform=False, suffix="_WithMotionRegressors"):
    """Add motion regressors to design matrix.

    This function will write a new .sdm file.

    Parameters
    ----------
    sdm : str
        filename of design matrix file
    motion : str
        filename of motion regressors file
    ztransform: bool. optional
        whether to z-transform the motion regressors (default=False)
    suffix : str, optional
        the suffix to add to the design matrix file (default="_WithMotionRegressors")

    """

    data = {"sdm": {"file": sdm},
            "motion": {"file": motion}}

    # Read design matrix and motion regressors files
    for x in data:
        with open(data[x]["file"]) as f:
            lines = f.readlines()
        for counter, line in enumerate(lines):
            if line.startswith('"'):
                break
        data[x]["names"] = line.strip().strip('"').split('" "')
        data[x]["colours"] = lines[counter-1].strip().split('   ')
        data[x]["header"] = [line.strip() for line in lines[:counter-1]]
        data[x]["data"] = np.array([[float(line[i:i+12]) for i in range(0, len(line), 12) if line[i]!="\n"]
                                    for line in lines[counter + 1:]])

    if ztransform:
        data["motion"]["data"] = zscore(data["motion"]["data"], axis=0)

    # Combine design matrix regressors with motion regressors
    data["motion"]["names"].append(data["sdm"]["names"].pop())
    data["motion"]["colours"].append(data["sdm"]["colours"].pop())
    data["motion"]["data"] = np.append(data["motion"]["data"], data["sdm"]["data"][:, -1:], axis=1)
    data["sdm"]["data"] = data["sdm"]["data"][:,:-1]

    data["sdm"]["names"].extend(data["motion"]["names"])
    data["sdm"]["colours"].extend(data["motion"]["colours"])
    data["sdm"]["data"] = np.append(data["sdm"]["data"], data["motion"]["data"], axis=1)

    for c, h in enumerate(data["sdm"]["header"]):
        if h.startswith("NrOfPredictors"):
            break
    data["sdm"]["header"][c] = h[:24] + repr(data["sdm"]["data"].shape[1])

    # Write new design matrix file
    path, ext = os.path.splitext(sdm)
    with open(path + "_" + suffix + ext, 'w') as f:
        for line in data["sdm"]["header"]:
            f.write(line + "\n")
        f.write("   ".join(data["sdm"]["colours"]) + "\n")
        f.write('"' + '" "'.join(data["sdm"]["names"]) + '"\n')
        for line in data["sdm"]["data"]:
            f.write("".join(["{0:f}".format(x).rjust(12) for x in line]) + "\n")
