"""Expand Motion Regressors script for BrainVoyager 20.

This script takes a .sdm file with motion regressors only (no constant)
and creates a new .sdm file that includes the original 6 movement regressors
plus their first derivative and optionally their second derivative. Derivatives
are calculated as  n - (n-1). Eventually, all regressors (original and
derivatives) can be z-transformed.

When the script is run from BrainVoyager, a GUI will be shown for all settings.

In addition, the script can be imported as a module in other scripts and used
like this:

    from expand_motion_regressors import MotionRegressorExpander

    mre = MotionRegressorExpander()
    mre.motion_file = "/path/to/3DMC_file.sdm"
    mre.expansions = 2
    mre.ztransform = True
    new_sdm_file = mre.expand()

"""

__author__ = "Florian Krause"
__version__ = "0.2.0"
__date__ = "2015-08-07"


import os

import PythonQt.Qt as qt
import numpy as np
from scipy.stats.mstats import zscore


class MotionRegressorExpander:
    """A class implementing a mechanism to expand motion regressors."""
    
    def __init__(self):
        
        self.motion_file = None
        self.expansions = 1
        self.ztransform = False

    def _load_file(self):
        """Load the motion file."""

        self.motion_file = BrainVoyager.BrowseFile("Select Motion File", "*.sdm")
        self._line.setText(self.motion_file)
        self._check_expand()
 
    def expand(self):
        """Expand the motion regressors."""

        # Read regressor file
        with open(self.motion_file) as f:
            lines = f.readlines()
        for counter, line in enumerate(lines):
            if line.startswith('"Translation BV-X [mm]"'):
                break
        names = line.strip().strip('"').split('" "')
        colours = lines[counter-1].strip().split('   ')
        motion_header = [line.strip() for line in lines[:counter-1]]
        motion_data = np.array([[float(line[i:i+12]) for i in range(0, len(line), 12) if line[i]!="\n"]
                                for line in lines[counter + 1:]])

        # Add derivative regressors
        for e in range(self.expansions):
            for r in range(6):
                for c, h in enumerate(motion_header):
                    if h.startswith("NrOfPredictors"):
                        break
                motion_header[c] = h[:25] + repr(int(h[-2:]) + 1)
                colours.append(colours[-6])   
                names.append(names[r] + " D{0}".format(e+1))
                motion_data = np.append(motion_data,
                                        np.ediff1d(motion_data[:,r+e*6], to_begin=0)[None].T,
                                        axis=1) 

        # Z-transform
        if self.ztransform:
            motion_data = zscore(motion_data, axis=0)

        # Write new regressor file
        path, ext = os.path.splitext(self.motion_file)
        add = "".join(["D{0}".format(e+1) for e in range(self.expansions)])
        with open(path + "_" + add + ext, 'w') as f:
            for line in motion_header:
                f.write(line + "\n")
            f.write("   ".join(colours) + "\n")
            f.write('"' + '" "'.join(names) + '"\n')
            for line in motion_data:
                f.write("".join(["{0:f}".format(x).rjust(12) for x in line]) + "\n")

        # Finish script
        self._dlg.close()
        BrainVoyager.PrintToLog("Expand Motion Regressors: {0} written!".format(
            path + "_" + add + ext))

        return path + "_" + add + ext
 
    def _check_expand(self):
        """Check whether all is set for expanding."""
        
        if self._check1.isChecked():
            self.expansions = 1
        elif self._check2.isChecked():
            self.expansions = 2
        if self._line.text != "":
            self._btn2.setEnabled(True)
        else:
            self._btn2.setEnabled(False)

    def _check_ztransform(self):
        if self._check3.isChecked():
            self.ztransform = False
        elif self._check4.isChecked():
            self.ztransform = True

    def show_gui(self):
        """Show the GUI dialogue."""

        self._dlg = qt.QDialog()
        self._dlg.resize(500, 100)
        self._dlg.windowTitle = 'Expand Motion Regressors ' + __version__
        self._box = qt.QVBoxLayout(self._dlg)

        self._group1 = qt.QGroupBox(self._dlg)
        self._group1.title = 'Motion File'
        self._box.addWidget(self._group1)
        self._box1 = qt.QHBoxLayout(self._group1)
        self._line = qt.QLineEdit(self._group1)
        self._line.readOnly = True
        self._box1.addWidget(self._line)
        self._btn1 = qt.QPushButton(self._group1)
        self._btn1.text = 'Select'
        self._btn1.connect('clicked(bool)', self._load_file)
        self._box1.addWidget(self._btn1)

        self._group2 = qt.QGroupBox(self._dlg)
        self._group2.title = 'Regressors'       
        self._box.addWidget(self._group2)
        self._box2 = qt.QVBoxLayout(self._group2)
        self._check1 = qt.QRadioButton(self._group2)
        self._box2.addWidget(self._check1)
        self._check1.text  = '1st derivative'
        self._check1.setChecked(2)
        self._check1.connect('clicked(bool)', self._check_expand)
        self._check2 = qt.QRadioButton(self._group2)
        self._box2.addWidget(self._check2)
        self._check2.text  = '1st derivative + 2nd derivative'
        self._check2.connect('clicked(bool)', self._check_expand)

        self._group3 = qt.QGroupBox(self._dlg)
        self._group3.title = 'Z-transformation'       
        self._box.addWidget(self._group3)
        self._box3 = qt.QHBoxLayout(self._group3)
        self._check3 = qt.QRadioButton(self._group3)
        self._box3.addWidget(self._check3)
        self._check3.text  = 'No'
        self._check3.setChecked(2)
        self._check3.connect('clicked(bool)', self._check_ztransform)
        self._check4 = qt.QRadioButton(self._group3)
        self._box3.addWidget(self._check4)
        self._check4.text  = 'Yes'
        self._check4.connect('clicked(bool)', self._check_ztransform)

        self._btn2 = qt.QPushButton(self._dlg)
        self._btn2.text = 'Expand'
        self._btn2.connect('clicked(bool)', self.expand)
        self._btn2.setEnabled(False)
        self._box.addWidget(self._btn2)

        self._dlg.show()


if __name__ == "__main__":
    MotionRegressorExpander().show_gui()